set echo on
set feedback on
set serveroutput on
var start_date  varchar2(80);
var end_date    varchar2(80);
var sid         varchar2(10);
var host        varchar2(80);
var user        varchar2(30);


spool 3.1.0_Update_Script.log;

begin
  select to_char(sysdate, 'mm-dd-yyyy hh24:mi:ss') into :start_date from dual;
  select instance_name into :sid from v$instance;
  select host_name into :host from v$instance;
  select user into :user from dual;

  dbms_output.put_line('Script executed on SID:  '||:sid);
  dbms_output.put_line('Script executed on host: '||:host);
  dbms_output.put_line('Script start date/time:  '||:start_date);
  dbms_output.put_line('Script executed by:      '||:user);
end;
/
SET DEFINE OFF;

CREATE OR REPLACE FUNCTION VAP.GETFRIENDLYSCHEDULEFREQUENCY 
(FREQUENCY VARCHAR2, 
 WEEKLY_RECURRENCE_WEEK NUMBER,
 WEEKLY_RECURRENCE_DAY_OF_WEEK NUMBER, 
 MONTHLY_RECURRENCE_MONTH NUMBER, 
 MONTHLY_RECURRENCE_DAY VARCHAR2
)
  RETURN VARCHAR2 DETERMINISTIC
IS 
V_RETURN VARCHAR2(300);
WEEK_DAY VARCHAR2(10);
MONTH_DAY VARCHAR(30);
BEGIN

  -- convert the WEEKLY_RECURRENCE_DAY_OF_WEEK from a number to the textual day of the week
  WEEK_DAY := 
    CASE  
      WHEN WEEKLY_RECURRENCE_DAY_OF_WEEK = 1 THEN 'Sunday'
      WHEN WEEKLY_RECURRENCE_DAY_OF_WEEK = 2 THEN 'Monday'
      WHEN WEEKLY_RECURRENCE_DAY_OF_WEEK = 3 THEN 'Tuesday'
      WHEN WEEKLY_RECURRENCE_DAY_OF_WEEK = 4 THEN 'Wednesday'
      WHEN WEEKLY_RECURRENCE_DAY_OF_WEEK = 5 THEN 'Thursday'
      WHEN WEEKLY_RECURRENCE_DAY_OF_WEEK = 6 THEN 'Friday'
      ELSE 'Saturday' 
    END;
  
  -- convert the MONTHLY_RECURRENCE_DAY to a readable day of the month
  MONTH_DAY := 
    CASE
      WHEN MONTHLY_RECURRENCE_DAY = 'first' THEN '1st day of the month'
      WHEN MONTHLY_RECURRENCE_DAY = 'fifteenth' THEN '15th of the month'
      ELSE 'last day of the month'
    END;

  IF FREQUENCY = 'DAILY' THEN
        V_RETURN := 'Daily';
    END IF;

    IF FREQUENCY = 'WEEKLY' THEN
        V_RETURN := 'Weekly recurring every ' || TO_CHAR(WEEKLY_RECURRENCE_WEEK) || ' week(s) on ' || WEEK_DAY;
    END IF;

    IF FREQUENCY = 'MONTHLY' THEN 
        V_RETURN := 'Monthly recurring every ' || TO_CHAR(MONTHLY_RECURRENCE_MONTH) || ' month(s) on the ' || MONTH_DAY;
    END IF;

    RETURN V_RETURN;

  END;
/

CREATE TABLE VAP.USER_DOCUMENT_SCHEDULE  ( 
    SCHEDULE_ID                     NUMBER NOT NULL,
    USER_ID                         VARCHAR2(255) NOT NULL,
    CREATION_DATE                   TIMESTAMP(6) NOT NULL,
    LAST_UPDATED_DATE               TIMESTAMP(6) NOT NULL,
    REPORT_TYPE                     VARCHAR2(255) NOT NULL,
    USER_DEFINED_REPORT_NAME        VARCHAR2(255) NOT NULL,
    FORMAT                          VARCHAR2(30) NOT NULL,
    PERIOD_OF_REPORTING             VARCHAR2(30) NOT NULL,
    REPORT_PARAMETERS               CLOB NULL,
    REPORT_VIEW_PARAMETERS          CLOB NULL,
    FREQUENCY                       VARCHAR2(10) NOT NULL, 
    WEEKLY_RECURRENCE_WEEK          NUMBER NULL, 
    WEEKLY_RECURRENCE_DAY_OF_WEEK   NUMBER NULL,
    MONTHLY_RECURRENCE_MONTH        NUMBER NULL, 
    MONTHLY_RECURRENCE_DAY          VARCHAR2(10) NULL, 
    START_DATE                      TIMESTAMP(6) NOT NULL,
    END_DATE                        TIMESTAMP(6) NULL,
    FREQUENCY_TEXT                  VARCHAR2(4000) GENERATED ALWAYS AS (VAP.GETFRIENDLYSCHEDULEFREQUENCY(FREQUENCY, WEEKLY_RECURRENCE_WEEK, WEEKLY_RECURRENCE_DAY_OF_WEEK, MONTHLY_RECURRENCE_MONTH, MONTHLY_RECURRENCE_DAY)),
    CONSTRAINT USER_DOCUMENT_SCHEDULE_PK PRIMARY KEY(SCHEDULE_ID)
);

CREATE TABLE VAP.USER_DOCUMENT_SCHEDULE_AUDIT (
    AUDIT_ID        NUMBER NOT NULL, 
    SCHEDULE_ID     NUMBER NOT NULL, 
    RUN_START_DATE  TIMESTAMP(6) NOT NULL, 
    RUN_END_DATE    TIMESTAMP(6) NULL, 
    STATUS          VARCHAR2(30) NOT NULL,
    ERROR_MESSAGE   CLOB NULL, 
    FORMAT          VARCHAR2(30) NOT NULL, 
	CHECK_DATE VARCHAR2(10) DEFAULT (TRUNC(SYSDATE)) NOT NULL, 
    CONSTRAINT USER_DOC_SCHEDULE_AUDIT_PK PRIMARY KEY(AUDIT_ID)
);

ALTER TABLE VAP.USER_DOCUMENT_SCHEDULE_AUDIT
	ADD ( CONSTRAINT UDS_SI_FK
	FOREIGN KEY(SCHEDULE_ID)
	REFERENCES VAP.USER_DOCUMENT_SCHEDULE(SCHEDULE_ID)
  ON DELETE CASCADE);

ALTER TABLE VAP.USER_DOCUMENT_SCHEDULE_AUDIT
ADD CONSTRAINT UDSA_SCHEDULE_ID_CHECK_DATE UNIQUE (SCHEDULE_ID, CHECK_DATE);  
  
CREATE SYNONYM VAP_APP.USER_DOCUMENT_SCHEDULE FOR VAP.USER_DOCUMENT_SCHEDULE;

GRANT SELECT ON VAP.USER_DOCUMENT_SCHEDULE TO VAP_READ;

GRANT DELETE, INSERT, SELECT, UPDATE ON VAP.USER_DOCUMENT_SCHEDULE TO VAP_USER;

CREATE SYNONYM VAP_APP.USER_DOCUMENT_SCHEDULE_AUDIT FOR VAP.USER_DOCUMENT_SCHEDULE_AUDIT;

GRANT SELECT ON VAP.USER_DOCUMENT_SCHEDULE_AUDIT TO VAP_READ;

GRANT DELETE, INSERT, SELECT, UPDATE ON VAP.USER_DOCUMENT_SCHEDULE_AUDIT TO VAP_USER;  
  
-- inserts for all advanced scheduling related descriptors (tooltips)
INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingReportName', 'This form field is a user-input field used to capture the name of an advanced scheduled reporting task. An example of this could be "Patient Discovery Audit Weekly Export Task."');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingReportingPeriod', 'This dropdown allows the user to specify the time frame of data that should be included in the export. For example, this be include data from the past day, past week, past month.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingFrequencyWeeklyWeeks', 'This form field specifies the recurrence of this advanced scheduled task to be run on a weekly basis.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingFrequencyWeeklyDayOfWeek', 'This dropdown specifies the recurrence of this advanced scheduled task to be run on every designated week on a certain day of the week. For example, this task can be set to run every two weeks on a Sunday.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingFrequencyMonthlyMonths', 'This form field specifies the recurrence of this advanced scheduled task to be run on a monthly basis.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingFrequencyMonthlyDayOfMonth', 'This dropdown specifies the recurrence of this advanced scheduled task to be run on a certain day of the month. For example, this task can be set to run every one month on the 1st day of the month.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingEndDateAfterOccurrences', 'This form field requires the user to set an end for the recurrence of this advance scheduling task. Users must enter a numerical number between 1 and 30.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingEndBy', 'This form field requires the user to set an end date for the recurrence of this advance scheduling task. Users may not schedule an advanced task to continue past six months.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingSelectAllColumn', 'This checkbox allows the users to select all the checkboxes for the rows listed on this page at one time.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingNameColumn', 'This column displays the user-entered name of an advanced scheduled reporting task.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingTypeColumn', 'This column displays the report type for the associated advanced scheduled reporting task. For example, this may specify this task is associated to the Received eHealth Exchange Detailed report.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingFormatColumn', 'This column displays the format of the advanced scheduled report export. Files can be exported in either Excel format or CSV format.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingDateCreatedColumn', 'This column displays the timestamp the advanced scheduled task was created by a user. All times are recorded in the Veterans Authorization and Preferences (VAP) system using Central Standard Time (CST).');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingStartDateColumn', 'This column displays the start date of the advanced scheduled task.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingEndDateColumn', 'This column displays the date advanced scheduled task is set to end.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingFrequencyColumn', 'This column displays the frequency of this advanced scheduled task. A user is able to set a task to occur daily, weekly, monthly or a recurring specified setting.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingLogStatusColumn', 'This column displays the status either completed, in progress, or failed of an advanced scheduled task.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingLogFormatColumn', 'This column displays the format of the advanced scheduled report export. Files can be exported in either Excel format or CSV format.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingLogRunStartDateColumn', 'This column displays the date and time of when the advanced scheduled task started.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'advancedSchedulingLogRunEndDateColumn', 'This column displays the date and time of when the advanced scheduled task completed.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'optInPatientsReportEntryDateColumn', 'This column displays the date a consent was entered into the VAP system.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'batchAnnouncePatientsOrganzationNameColumn', 'This field displays the organization or partner to whom the announcement will be sent to.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'batchAnnouncePatientsPatientCountColumn', 'This field displays the total number of unique patients that are included in the announcement to that organization. Announcements are made to share patient identifiers, such as ICNs, with partner systems to facilitate data sharing.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'batchAnnounceDetailsPatientIENColumn', 'This column displays the IEN associated to the individual included in the announcement.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'batchAnnounceDetailsSSNColumn', 'This column displays the Social Security Number (SSN) for individual included in the announcement.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'batchAnnounceDetailsLastNameColumn', 'This column displays the last name of the individual who was included in the announcement.');

INSERT INTO VAP.DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(VAP.HIBERNATE_SEQUENCE.NEXTVAL, 'batchAnnounceDetailsFirstNameColumn', 'This column displays the first name of the individual who was included in the announcement.');

UPDATE VAP.DESCRIPTOR
SET TEXT = 'This dropdown allows the user to specify the timeframe of data that should be included in the export. For example, this may include data from the past day, past week, or past month.'
WHERE NAME = 'advancedSchedulingReportingPeriod';

COMMIT;
  
SET DEFINE ON;

Begin
  select to_char(sysdate, 'mm-dd-yyyy hh24:mi:ss') into :end_date from dual;
  dbms_output.put_line('Script end date/time:    '||:end_date);
  dbms_output.put_line('Script executed in '||(to_date(:end_date, 'mm_dd_yyyy hh24:mi:ss') - to_date(:start_date, 'mm-dd-yyyy hh24:mi:ss')) *24*60*60||' seconds');
end;
/

Spool off;
