spool VAP_Update2.7.log

ALTER SESSION SET CURRENT_SCHEMA = VAP;

--------------------------------------------------------
--  ADD NEW COLUMNS TO FACILITY TABLE
--------------------------------------------------------

ALTER TABLE FACILITY
ADD (
  FACILITY_PARENT VARCHAR2(20 BYTE),
  ACTIVE VARCHAR2(20 BYTE)
);

COMMIT;

--------------------------------------------------------
--  ADD NEW COLUMN TO ALLOWED_ORG TABLE
--------------------------------------------------------

ALTER TABLE ALLOWED_ORG
ADD (
  ORG_DESCRIPTION VARCHAR2(255 BYTE)
);

COMMIT;

--------------------------------------------------------
--  UPDATE CONSENT_AUDIT_EX VIEW
--------------------------------------------------------

CREATE OR REPLACE FORCE VIEW CONSENT_AUDIT_EX
(
  CONSENT_AUDIT_ID, ACTION_TYPE, CONSENT_TYPE, CREATED_DATE, FACILITY, FACILITY_NAME, OPTOUT_REASON,
  PATIENT_GIVEN_NAME, PATIENT_ID, PATIENT_LAST_NAME, PATIENT_SSN, POU_VALUE, TIME_OF_EVENT, USER_ID,
  PATIENT_MIDDLE_NAME, VISN_NAME, VISN_NUMBER, PARENT_STATION_NUMBER, IS_TEST_PATIENT, PARENT_FACILITY_NAME
) AS
select c.CONSENT_AUDIT_ID, c.ACTION_TYPE, c.CONSENT_TYPE, c.CREATED_DATE,
         nvl(fa.FACILITY_STATION, c.FACILITY) as FACILITY,
         nvl(fa.FACILITY_NAME, 'Unknown') as FACILITY_NAME,
         c.OPTOUT_REASON, c.PATIENT_GIVEN_NAME, c.PATIENT_ID,
         c.PATIENT_LAST_NAME, c.PATIENT_SSN, c.POU_VALUE, c.TIME_OF_EVENT,
         c.USER_ID, c.PATIENT_MIDDLE_NAME, v.VISN_NAME, v.VISN_NUMBER,
         case when fa.FACILITY_PARENT is null then fa.FACILITY_STATION else fa.FACILITY_PARENT end as PARENT_STATION_NUMBER,
         ISTESTPATIENT(c.PATIENT_SSN),
         nvl(fb.FACILITY_NAME, 'Unknown') PARENT_FACILITY_NAME
from (
  select ci.CONSENT_AUDIT_ID, ci.ACTION_TYPE, ci.CONSENT_TYPE, ci.CREATED_DATE,
         ci.FACILITY, ci.FACILITY_NAME, ci.OPTOUT_REASON, ci.PATIENT_GIVEN_NAME, ci.PATIENT_ID,
         ci.PATIENT_LAST_NAME, ci.PATIENT_SSN, ci.POU_VALUE, ci.TIME_OF_EVENT,
         ci.USER_ID, ci.PATIENT_MIDDLE_NAME,
         (select fi.FACILITY_ID from FACILITY fi where ci.FACILITY = fi.FACILITY_STATION) STATION_LOOKUP_TYPE,
         (select fi.FACILITY_ID from FACILITY fi where substr(ci.FACILITY,1,3) = fi.FACILITY_STATION) FACILITY_LOOKUP_TYPE
  from CONSENT_AUDIT ci
) c
left outer join FACILITY fa on fa.FACILITY_ID =
     case
       when c.STATION_LOOKUP_TYPE is not null then c.STATION_LOOKUP_TYPE
       when c.FACILITY_LOOKUP_TYPE is not null then c.FACILITY_LOOKUP_TYPE
     end
left outer join FACILITY fb on case when fa.FACILITY_PARENT is null then fa.FACILITY_STATION else fa.FACILITY_PARENT end = fb.FACILITY_STATION
left outer join VISN v on fa.VISN_ID = v.ID;

COMMIT;

--------------------------------------------------------
--  UPDATE DELAYED_CONSENT_VIEW VIEW
--------------------------------------------------------

CREATE OR REPLACE FORCE VIEW DELAYED_CONSENT_VIEW
(
  DELAYED_CONSENT_ID, PATIENT_IEN, PATIENT_SSN, PATIENT_LAST_NAME, PATIENT_FIRST_NAME, PATIENT_MIDDLE_NAME, DATE_ADDED, USER_ID, CONSENT_TYPE_ID,
  PARENT_STATION_NUMBER, STATION_NUMBER, FACILITY_NAME, STATUS, RESOLUTION_DATE, IS_TEST_PATIENT, PARENT_FACILITY_NAME, VISN_NUMBER
) AS
  select DELAYED_CONSENT_ID, PATIENT_IEN, PATIENT_SSN, PATIENT_LAST_NAME, PATIENT_FIRST_NAME, PATIENT_MIDDLE_NAME, DATE_ADDED, USER_ID,
       CONSENT_TYPE_ID, case when fa.FACILITY_PARENT is null then fa.FACILITY_STATION else fa.FACILITY_PARENT end as PARENT_STATION_NUMBER,
       STATION_NUMBER, fa.FACILITY_NAME, STATUS, RESOLUTION_DATE, ISTESTPATIENT(PATIENT_SSN), fb.FACILITY_NAME as PARENT_FACILITY_NAME, VISN_NUMBER
from DELAYED_CONSENT d
left outer join FACILITY fa on d.STATION_NUMBER = fa.FACILITY_STATION
left outer join FACILITY fb on case when fa.FACILITY_PARENT is null then fa.FACILITY_STATION else fa.FACILITY_PARENT end = fb.FACILITY_STATION
left outer join VISN v on fa.VISN_ID = v.ID;

COMMIT;

--------------------------------------------------------
--  CREATE NEW DESCRIPTORS
--------------------------------------------------------

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'partnerOrganizationDescriptionColumn', 'This column displays the partner organization description.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'partnerOrganizationDescriptionField', 'This field allows the admin to edit the partner organization description. This is a required field.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'manageBatchesOrganizationHeader', 'This column displays the organization name.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'manageBatchesDateHeader', 'This column displays the date created.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'manageBatchesScheduledHeader', 'This column displays the number of scheduled batches.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'manageBatchesCompletedHeader', 'This column displays the number of completed batches.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'manageBatchesSizeHeader', 'This column displays the batch size.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'manageBatchesActionHeader', 'This column contains the action button for the corresponding row.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'manageBatchesCheckBoxHeader', 'Checkbox that allows to Check all (one or many can be checked below)');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'disclosureSummaryReportFacilityIDColumn', 'This column displays the facility id of the preferred facility.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'viewFacilityStationNumber', 'This column displays the associated three or five digit Veteran''s Affairs facility number.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'viewFacilityName', 'This column displays the Veteran''s Affairs facility name.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'viewFacilityAddress', 'This column displays the facility address.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'viewFacilityCity', 'This column displays the facility city.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'viewFacilityState', 'This column displays the facility state.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'viewFacilityZipCode', 'This column displays facility zip code.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'viewFacilityPhoneNumber', 'This column displays the facility phone number.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'viewFacilityVISN', 'This column displays the VISN associated to the facility.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'viewFacilityParent', 'This column displays the parent of the associated facility. When selecting the aggregate button on summary reports, the totals of all the children facilities will be rolled up and added to Parent Facility counts. Relationship can be revised through the "Edit" action.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'viewFacilityChildren', 'This column displays the children of the associated facility. When selecting the aggregate button on summary reports, all of the counts from children facilities are rolled up and aggregated to Parent Facility counts. This relationship can be revised through the "Edit" action.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'viewFacilityAllowed', 'This column displays if a facility is allowed.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'viewFacilityAction', 'This column displays the available actions for the corresponding facility.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'editFacilityStationNumber', 'This field allows the user to edit the facility station number.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'editFacilityDNS', 'This field allows the user to edit the facility DNS.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'editFacilityName', 'This field allows the user to edit the facility name.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'editFacilityAddress', 'This field allows the user to edit the facility address.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'editFacilityCity', 'This field allows the user to edit the facility city.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'editFacilityState', 'This field allows the user to edi the facility state.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'editFacilityZipCode', 'This field allows the user to edit the facility zip code.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'editFacilityPhoneNumber', 'This field allows the user to edit the facility phone number.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'editFacilityVisn', 'This field allows the user to edit the facility visn.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'editFacilityParentList', 'This field allows the user to select a parent facility.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'editFacilityAllowed', 'This field allows the user to select if a facility is allowed or not.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'actionType', 'This field allows for filtering of the resultant data within this report to include the counts of only received (inbound) documents from partners or disclosed (outbound) to partners, or both.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'documentSizeOrgCol', 'This column displays the organizations from which VA has disclosed records or received records through eHealth Exchange.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'loincCode', 'This column displays the results type (LOINC code) for the associated documents exchanged.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'documentSizeTitle', 'This column displays the title of the documents exchanged.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'documentSizeCount', 'This column displays the total count of documents exchanged for the associated results type and document title within the selected dates. ');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'documentSizeAverageSize', 'This column displays the average file size of the documents exchanged (in kb) for the associated results type and document title.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'documentSizeMinimumSize', 'This column displays the file size of the smallest document (in kb) for the associated results type and document title.');

INSERT INTO DESCRIPTOR
(DESCRIPTOR_ID, NAME, TEXT)
VALUES
(HIBERNATE_SEQUENCE.NEXTVAL, 'documentSizeMaximumSize', 'This column displays the largest individual file size (in kb) for documents within the associated results type and document title.');

COMMIT;

--------------------------------------------------------
--  CREATE USER_ROLE SEQUENCE
--------------------------------------------------------

CREATE SEQUENCE vap.user_role_seq
  START WITH     1000
  INCREMENT BY   1
  NOCACHE
  NOCYCLE;

COMMIT;

--------------------------------------------------------
--  CREATE ALLOWED_ROLE TABLE
--------------------------------------------------------

CREATE TABLE "VAP"."ALLOWED_ROLE"
(
  "ROLE_ID" NUMBER NOT NULL,
  "ROLE_NAME" VARCHAR2(50 BYTE) NOT NULL,
  CONSTRAINT "ALLOWED_ROLE_PK" PRIMARY KEY ("ROLE_ID")
);

CREATE OR REPLACE SYNONYM VAP_APP.ALLOWED_ROLE FOR VAP.ALLOWED_ROLE;
GRANT SELECT ON VAP.ALLOWED_ROLE TO VAP_USER;
GRANT SELECT ON VAP.ALLOWED_ROLE TO VAP_READ;
GRANT INSERT ON VAP.ALLOWED_ROLE TO VAP_USER;
GRANT UPDATE ON VAP.ALLOWED_ROLE TO VAP_USER;
GRANT DELETE ON VAP.ALLOWED_ROLE TO VAP_USER;

COMMIT;

--------------------------------------------------------
--  POPULATE ALLOWED_ROLE TABLE
--------------------------------------------------------

insert into vap.allowed_role (role_id, role_name) values (1,'VAPApplicationWebAdministrator');
insert into vap.allowed_role (role_id, role_name) values (2,'VAPApplicationWebUser');
insert into vap.allowed_role (role_id, role_name) values (3,'VAPApplicationReportsUser');
insert into vap.allowed_role (role_id, role_name) values (4,'VAPApplicationTester');

COMMIT;

--------------------------------------------------------
--  CREATE USER_ROLE TABLE
--------------------------------------------------------

CREATE TABLE "VAP"."USER_ROLE"
(
  "USER_ROLE_ID" NUMBER NOT NULL ENABLE,
  "USER_NAME" VARCHAR2(50 BYTE) NOT NULL ENABLE,
  "USER_ROLE" NUMBER,
  CONSTRAINT "USER_ROLE_PK" PRIMARY KEY ("USER_ROLE_ID") ENABLE,
  CONSTRAINT "USER_ROLE_FK" FOREIGN KEY ("USER_ROLE")
  REFERENCES "VAP"."ALLOWED_ROLE" ("ROLE_ID") ENABLE
);

CREATE OR REPLACE SYNONYM VAP_APP.USER_ROLE FOR VAP.USER_ROLE;
GRANT SELECT ON VAP.USER_ROLE TO VAP_USER;
GRANT SELECT ON VAP.USER_ROLE TO VAP_READ;
GRANT INSERT ON VAP.USER_ROLE TO VAP_USER;
GRANT UPDATE ON VAP.USER_ROLE TO VAP_USER;
GRANT DELETE ON VAP.USER_ROLE TO VAP_USER;

COMMIT;