package gov.va.nvap.web.facility;

import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.web.dao.FacilityDAO;
import gov.va.nvap.web.helper.report.ReportHelper;
import gov.va.nvap.web.user.User;
import gov.va.nvap.web.util.xls.ExcelExporter;
import java.io.IOException;
import java.io.StringWriter;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.poi.ss.usermodel.Workbook;
import org.json.JSONException;
import org.json.JSONObject;

/**
 *
 * @author Raul Alfaro
 */
public class ViewFacility extends gov.va.nvap.web.app.ResponseDispatcherHttpServlet {
    
    private static final long serialVersionUID = 1L;

    @Override
    protected void unspecified(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        final HttpSession session = request.getSession(false);
        session.removeAttribute("results");
        this.forward(request, response, "show");
    }

    public void doSearch(final HttpServletRequest request, final HttpServletResponse response) throws ServletException, IOException {
        final HttpSession session = request.getSession(false);
        session.setAttribute("results", encodeToJSON(getResults(request)));

        this.forward(request, response, "searchResultsJSON");
    }

    public HashMap<String, Object> getResults(HttpServletRequest request) {
        String sortColumn = ""; // getSortValue((request.getParameter("order[0][column]") != null) ? request.getParameter("order[0][column]") : "1");
        String sortDirection = (request.getParameter("order[0][dir]") != null) ? request.getParameter("order[0][dir]") : "asc";
        final String maxRows = !NullChecker.isNullOrEmpty(request.getParameter("length")) ? request.getParameter("length") : "-1";
        final String startRow = !NullChecker.isNullOrEmpty(request.getParameter("start")) ? request.getParameter("start") : "0";
        final HttpSession session = request.getSession(false);
        User u = (User) session.getAttribute("user");
        
        //handle user preferences for this report
        if("1".equals(request.getParameter("draw"))) {
            //if its the first time we're running this report on this page, we need to check the user's preference
            if(u.hasUserPreference("reportFacilitySortColumn")) {
                sortColumn = getSortValue(u.getUserPreference("reportFacilitySortColumn"));
                sortDirection = u.getUserPreference("reportFacilitySortDirection");
            } else if(request.getParameter("order[0][column]") != null) {
                sortColumn = getSortValue(request.getParameter("order[0][column]"));
                sortDirection = request.getParameter("order[0][dir]");
                u.addUserPreference("reportFacilitySortColumn", request.getParameter("order[0][column]"));
                u.addUserPreference("reportFacilitySortDirection", request.getParameter("order[0][dir]"));
            } else {
                sortColumn = getSortValue("1");
                sortDirection = "asc";
                u.addUserPreference("reportFacilitySortColumn", "1");
                u.addUserPreference("reportFacilitySortDirection", "asc");
            }
        } else {
            //otherwise we use what is sent and set it into the user's preference
            sortColumn = getSortValue((request.getParameter("order[0][column]") != null) ? request.getParameter("order[0][column]") : "1");
            
            u.addUserPreference("reportFacilitySortColumn", ((request.getParameter("order[0][column]") != null) ? request.getParameter("order[0][column]") : "1"));
            u.addUserPreference("reportFacilitySortDirection", request.getParameter("order[0][dir]"));
        }
        
        FacilityDAO facilityDao = getFacilityDAO();
        List<FacilityInfo> results = facilityDao.findFacilityInfo(sortColumn, sortDirection, Integer.parseInt(maxRows), Integer.parseInt(startRow));
        Integer count = facilityDao.findFacilityInfo(sortColumn, sortDirection, -1, 0).size(); //second call here is for the count of total facilities so DataTables knows how to do paging
        
        //return blank list if no results were found. Should never happen but just in case.
        HashMap<String, Object> hm = new HashMap<String, Object>();
        hm.put("results", results);
        hm.put("totalCount", count);
        
        return hm;
    }

    public String encodeToJSON(HashMap<String, Object> hm) {
        String returnValue = "";
        List<FacilityInfo> rawResults = (List<FacilityInfo>) hm.get("results");
        Integer totalCount = (Integer) hm.get("totalCount");
        
        try {
            StringWriter json = new StringWriter();
            JSONObject obj = new JSONObject();
            List<List<String>> data = new ArrayList<List<String>>();

            for (FacilityInfo current : rawResults) {
                List<String> dataItem = new ArrayList<>();

                dataItem.add(current.getFacilityStation());
                dataItem.add(current.getFacilityName());
                dataItem.add(current.getAddress());
                dataItem.add(current.getCity());
                dataItem.add(current.getState());
                dataItem.add(current.getPostalCode());
                dataItem.add(ReportHelper.normalizePhoneNumber(current.getPhone()));
                dataItem.add((current.getVisnId() != null) ? current.getVisnId().toString() : "");
                dataItem.add((current.getParentStationNumber() != null) ? current.getParentName() + " (" + current.getParentStationNumber() + ")" : "None");
                dataItem.add((current.getChildren() != null) ? current.getChildren() : "None");
                dataItem.add((current.isAllowed()) ? "Yes" : "No");
                dataItem.add((current.getActive()!= null) ? current.getActive() : "True");
                data.add(dataItem);
            }
            obj.put("data", data);
            obj.put("recordsTotal", totalCount);
            obj.put("recordsFiltered", totalCount);
            obj.write(json);

            returnValue = json.toString();
        } catch (JSONException ex) {
            Logger.getLogger(ViewFacility.class.getName()).log(Level.SEVERE, null, ex);
        }

        return returnValue;
    }

    public void exportToExcel(final HttpServletRequest request,
        final HttpServletResponse response) throws ServletException {

        // Create a map of key which is based on the result data key and the heading value
        //The heading is used to create the column headers and the
        //key is used to pull the data from the results
        final Map<String, String> reportMap = new LinkedHashMap<>();

        // Set the heading parameters 
        final Map<String, List<Object>> headingParameters = new LinkedHashMap<>();

        reportMap.put("stationNumber", "Station Number");
        reportMap.put("facilityName", "Name");
        reportMap.put("facilityAddress", "Address");
        reportMap.put("facilityCity", "City");
        reportMap.put("facilityState", "State");
        reportMap.put("facilityPostalCode", "Zip Code");
        reportMap.put("facilityPhone", "Phone Number");
        reportMap.put("facilityVISN", "VISN");
        reportMap.put("facilityParent", "Parent");
        reportMap.put("facilityChildren", "Children");
        reportMap.put("allowed", "Allowed");

        // Create workbook
        final String title = "Facilities";

        List<Map<String, Object>> results = transformToMap((List<FacilityInfo>) this.getResults(request).get("results"));

        final Workbook wb = this.getExcelExporter().exportToExcel(title, title, reportMap, results, headingParameters, null);

        // Write Excel workbook to Stream
        this.getExcelExporter().writeExcelToStream("Facilities", wb, response);
    }

    public List<Map<String, Object>> transformToMap(List<FacilityInfo> facilities) {
        List<Map<String, Object>> result = new ArrayList<>();

        for (FacilityInfo current : facilities) {
            Map<String, Object> dataItem = new HashMap<>();

            /*String phone = current.getPhone();
            if (!NullChecker.isNullOrEmpty(phone)) {
                phone = "(" + phone.substring(0, 3) + ") " + phone.substring(3, 6) + "-" + phone.substring(6);
            }*/
            
            dataItem.put("stationNumber", current.getFacilityStation());
            dataItem.put("facilityName", current.getFacilityName());
            dataItem.put("facilityAddress", current.getAddress());
            dataItem.put("facilityCity", current.getCity());
            dataItem.put("facilityState", current.getState());
            dataItem.put("facilityPostalCode", current.getPostalCode());
            dataItem.put("facilityPhone", ReportHelper.normalizePhoneNumber(current.getPhone())); 
            dataItem.put("facilityVISN", (current.getVisnId() != null) ? "VISN " + current.getVisnId().toString() + "- " + current.getVisnName() : "");
            dataItem.put("facilityParent", (current.getParentStationNumber() != null) ? current.getParentName() + " (" + current.getParentStationNumber() + ")" : "None");
            dataItem.put("facilityChildren", (current.getChildren() != null) ? current.getChildren() : "None");
            dataItem.put("allowed", (current.isAllowed()) ? "Yes" : "No");

            result.add(dataItem);
        }

        return result;
    }

    public String getSortValue(String colNumber) {
        String colName = "";
        switch (colNumber) {
            case "0":
                colName = "FACILITY_STATION";
                break;
            case "1":
                colName = "FACILITY_NAME";
                break;
            case "2":
                colName = "FACILITY_ADDRESS";
                break;
            case "3":
                colName = "FACILITY_CITY";
                break;
            case "4":
                colName = "FACILITY_STATE";
                break;
            case "5":
                colName = "FACILITY_POSTAL_CODE";
                break;
            case "6":
                colName = "FACILITY_PHONE";
                break;
            case "7":
                colName = "VISN_NUMBER"; //we display name but want to order by the VISN number
                break;
            default:
                colName = "FACILITY_NAME";
                break;
        }
        //Other columns left out are not sortable 
        return colName;
    }

    private FacilityDAO getFacilityDAO() {
        return this.getBean("FacilityDAO", FacilityDAO.class);
    }

    /**
     * Get the excel exporter class from Spring.
     *
     * @return the excel exporter object
     */
    public ExcelExporter getExcelExporter() {
        final ExcelExporter excelExporter = this.getBean("excelExporter", ExcelExporter.class);
        return excelExporter;
    }

}
