package gov.va.nvap.web.report;

import gov.va.med.nhin.adapter.audit.ActionType;
import gov.va.med.nhin.adapter.audit.ActionValuesType;
import gov.va.med.nhin.adapter.audit.DocumentAuditType;
import gov.va.med.nhin.adapter.audit.GetDocumentAuditsResponseType;
import gov.va.med.nhin.adapter.audit.GetDocumentAuditsType;
import gov.va.med.nhin.adapter.audit.StringValuesType;
import gov.va.nvap.common.date.GregorianDateUtil;
import gov.va.nvap.common.sort.BubbleSortListMap;
import gov.va.nvap.common.validation.NullChecker;
import gov.va.nvap.service.adapter.audit.AdapterAuditManager;
import gov.va.nvap.web.helper.privacy.ConsentManagementHelper;
import gov.va.nvap.web.helper.report.ReportHelper;
import gov.va.nvap.web.util.xls.ExcelExporter;
import java.io.IOException;
import java.io.StringWriter;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Date;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.ejb.EJB;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.http.HttpSession;
import org.apache.poi.ss.usermodel.Workbook;
import org.json.JSONException;
import org.json.JSONObject;

/**
 *
 * @author Zachary Tubb
 */
public class DocumentSizeReport extends gov.va.nvap.web.app.ResponseDispatcherHttpServlet {
    
    private static final long serialVersionUID = 1L;

    private AdapterAuditManager adapterAuditManager;

    public void doSearch(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException, ParseException {
        final HttpSession session = request.getSession(false);
        session.setAttribute("results", encodeIntoJSON(getResults(request)));

        this.forward(request, response, "searchResultsJSON");
    }

     private String encodeIntoJSON(SearchResultsPackage searchResults) {
        String jsonString = "";

        try {
            StringWriter json = new StringWriter();
            JSONObject obj = new JSONObject();
            List<List<String>> data = new ArrayList<List<String>>();
            int count = 0;

            for (Map<String, Object> row : searchResults.getSearchResults()) {
                List<String> dataItem = new ArrayList<String>();

                for (String key : row.keySet()) {
                	if (key.toLowerCase().contains("ssn")) {
                		dataItem.add(ReportDataProcessor.decryptToString(row.get(key)));
                	}else{
                		dataItem.add(row.get(key).toString());
                	}
                }
                data.add(dataItem);
                count++;
            }

            obj.put("data", data);
            obj.put("recordsTotal", count);

            obj.write(json);

            jsonString = json.toString();
        } catch (JSONException ex) {
            Logger.getLogger(PatientDiscoverySummaryReport.class.getName()).log(Level.SEVERE, null, ex);
        }
        return jsonString;
    }

    public void exportToExcel(final HttpServletRequest request, final HttpServletResponse response) throws ServletException,
        IOException {

        SearchResultsPackage results = getResults(request);

        // Create a map of key which is based on the result data key and the
        // heading value
        // The heading is used to create the column headers and the key is used
        // to pull the data from the results
        final Map<String, String> documentSizeMap = new LinkedHashMap<String, String>();
        documentSizeMap.put("organizationName", "eHealth Exchange Organization");
        documentSizeMap.put("loincCode", "LOINC Code");
        documentSizeMap.put("documentTitle", "Title");
        documentSizeMap.put("documentCount", "Number of Documents");
        documentSizeMap.put("averageSize", "Average Size");
        documentSizeMap.put("minimumSize", "Minimum Size");
        documentSizeMap.put("maximumSize", "Maximum Size");

        // Generate filters.
        final Map<String, List<Object>> filterMap = new LinkedHashMap<String, List<Object>>();
        LinkedHashMap<String, Object> filters = new LinkedHashMap<String, Object>();

        String orgName = request.getParameter("organizationName");
        if (NullChecker.isNullOrEmpty(orgName)) {
            orgName = "ALL";
        }
        String action = request.getParameter("action").toUpperCase();
        if (NullChecker.isNullOrEmpty(action)) {
            action = "ALL";
        }
        if(action.toUpperCase(Locale.ENGLISH).equals("INBOUND")){
            action = "Received Documents";
        } else if(action.toUpperCase(Locale.ENGLISH).equals("OUTBOUND")){
            action = "Disclosed Documents";
        }

        filters.put("Start Date", request.getParameter("startDate"));
        filters.put("End Date",request.getParameter("endDate"));
        filters.put("eHealth Exchange Organization", orgName);
        filters.put("Action", action);
        ExcelExporter.populateFilterMapForExport(request, filters, filterMap, "");

        // Create the optional data rows
        Map<String, List<Object>> optionalRows = null;
        final List<Object> documentSizeRow;

        // Do not add if empty
        if(results.getTotalCount() > 0){

            optionalRows = new LinkedHashMap<String, List<Object>>();
            documentSizeRow = new ArrayList<Object>();
            documentSizeRow.add("");
            documentSizeRow.add("");
            documentSizeRow.add(results.getTotalCount());

            optionalRows.put("Total", documentSizeRow);
        }
        // Create the workbook
        final String title = "Document Size Report";
        final Workbook wb = this.getExcelExporter().exportToExcel(title, title, documentSizeMap, results.getSearchResults(), filterMap, optionalRows);

        // Write Excel to Stream
        this.getExcelExporter().writeExcelToStream("Document_Size_Report", wb, response);
    }

    private ConsentManagementHelper getCmsHelper() {
        return this.getBean("cmsHelper", ConsentManagementHelper.class);
    }

    private ExcelExporter getExcelExporter() {
        return this.getBean("excelExporter", ExcelExporter.class);
    }

    private DocumentSizeReport.SearchResultsPackage getResults(final HttpServletRequest request)
            throws ServletException {

        //Get filters from request
        final Date sd = ReportHelper.getStartDate(request.getParameter("startDate"));
        final Date ed = ReportHelper.getEndDate(request.getParameter("endDate"));
        final String organization = request.getParameter("organization");
        final String action = request.getParameter("action");

        int count = 0;

        //Logic for building adapter request and making webservice call goes here.
        final GetDocumentAuditsType getDocumentsAuditRequest = new GetDocumentAuditsType();
        if (!NullChecker.isNullOrEmpty(sd)) {
            getDocumentsAuditRequest.setFromDate(GregorianDateUtil.getGregorianCalendarByDate(sd));
        }
        if (!NullChecker.isNullOrEmpty(ed)) {
            getDocumentsAuditRequest.setToDate(GregorianDateUtil.getGregorianCalendarByDate(ed));
        }

        // Set organization/partner/facility filter
        if (NullChecker.isNotEmpty(organization)) {
            final StringValuesType organizationIds = new StringValuesType();
            organizationIds.setNotIn(false);
            organizationIds.getValue().add(organization.replace("%", "urn:oid:"));
            getDocumentsAuditRequest.setOrganizationIds(organizationIds);
        } else {
            //Exclude VA results if searching for Disclosure documents
            if ("OUTBOUND".equals(action.toUpperCase(Locale.ENGLISH))) {
                final StringValuesType organizationIds = new StringValuesType();
                organizationIds.setNotIn(true);
                organizationIds.getValue().add(this.getCmsHelper().getCompleteHomeCommunityId());
                getDocumentsAuditRequest.setOrganizationIds(organizationIds);
            }
        }

        // Set action filter - if param is neither INBOUND nor OUTBOUND, we do not set
        // the action type so that the query will return both
        final ActionValuesType actionsType = new ActionValuesType();
        if ("INBOUND".equals(action.toUpperCase(Locale.ENGLISH))) {
            actionsType.getValue().add(ActionType.RETRIEVE_DOCUMENT_OUT);
            actionsType.setNotIn(false);
            getDocumentsAuditRequest.setActions(actionsType);
        } else if ("OUTBOUND".equals(action.toUpperCase(Locale.ENGLISH))) {
            actionsType.getValue().add(ActionType.RETRIEVE_DOCUMENT);
            actionsType.setNotIn(false);
            getDocumentsAuditRequest.setActions(actionsType);
        }

        final List<Map<String, Object>> results = new ArrayList<Map<String, Object>>();
        final GetDocumentAuditsResponseType queryResponse;

        queryResponse = this.adapterAuditManager.getDocumentAudits(getDocumentsAuditRequest);

        if (!NullChecker.isNullOrEmpty(queryResponse) && !NullChecker.isNullOrEmpty(queryResponse.getAudits())) {
            final List<DocumentAuditType> documentAudits = queryResponse.getAudits().getAudit();
            for (DocumentAuditType docAudit : documentAudits) {
                final Map<String, Object> resultMap = new LinkedHashMap<String, Object>();
                resultMap.put("organizationName", ReportDataProcessor.nullEmptyReplaceWithUnknown(docAudit.getOrganizationName()).toUpperCase());
                resultMap.put("loincCode", ReportDataProcessor.nullEmptyReplaceWithUnknown(docAudit.getClassCode()));
                resultMap.put("documentTitle", ReportDataProcessor.nullEmptyReplaceWithUnknown(docAudit.getTitle()));
                resultMap.put("documentCount", docAudit.getNumberOfDocs());
                resultMap.put("averageSize", Math.round(docAudit.getAverageSize() / 1024));
                resultMap.put("minimumSize", Math.round(Double.parseDouble(docAudit.getMinimumSize()) / 1024));
                resultMap.put("maximumSize", Math.round(Double.parseDouble(docAudit.getMaximumSize()) / 1024));
                results.add(resultMap);

                if (!NullChecker.isNullOrEmpty(docAudit.getNumberOfDocs())) {
                    count = (int) (count + docAudit.getNumberOfDocs());
                }
            }
        }

        return new SearchResultsPackage(sortResults(results,request), count);
    }

    private ReportHelper getReportHelper() {
        return this.getBean("reportHelper", ReportHelper.class);
    }

    @EJB(beanInterface = AdapterAuditManager.class, mappedName = "AdapterAuditManager")
    public void setAdapterAuditManager(final AdapterAuditManager adapterAuditManager) {
        this.adapterAuditManager = adapterAuditManager;
    }

    /**
     * sorts values of List<Map<String,Object>> keys based on request parameters
     * @param unsortedList
     * @param request
     * @return
     */
    private List<Map<String,Object>> sortResults(List<Map<String,Object>> unsortedList, HttpServletRequest request){
        if(unsortedList.isEmpty()){
            return unsortedList;
        }
        String sortValue = request.getParameter("order[0][column]") == null ? request.getParameter("sortBy") : request.getParameter("order[0][column]");
        boolean sortAscending = request.getParameter("order[0][dir]") == null ? !"desc".equalsIgnoreCase(request.getParameter("sortOrder")) :
            !"desc".equalsIgnoreCase(request.getParameter("order[0][dir]"));
        final BubbleSortListMap bSorter = new BubbleSortListMap();

        if("0".equals(sortValue)){
            sortValue = "organizationName";
        } else if ("1".equals(sortValue)){
            sortValue = "loincCode";
        } else if ("2".equals(sortValue)){
            sortValue = "documentTitle";
        } else if ("3".equals(sortValue)){
            sortValue = "documentCount";
        } else if ("4".equals(sortValue)) {
            sortValue = "averageSize";
        } else if ("5".equals(sortValue)) {
            sortValue = "minimumSize";
        } else if ("6".equals(sortValue)) {
            sortValue = "maximumSize";
        } else {
            sortValue = "organizationName";
        }
        return bSorter.sortByColumn(unsortedList, sortValue, sortAscending);
    }

    protected void unspecified(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
        final HttpSession session = request.getSession(false);
        session.removeAttribute("results");
        getReportHelper().getOrgLists(request, this.getCmsHelper());
        this.forward(request, response, "searchForm");
    }

    private class SearchResultsPackage {

        private final List<Map<String, Object>> searchResults;
        private final int count;

        public SearchResultsPackage(List<Map<String, Object>> searchResults, int count) {
            this.searchResults = searchResults;
            this.count = count;
        }

        public List<Map<String, Object>> getSearchResults() {
            return searchResults;
        }

        public int getTotalCount() {
            return count;
        }

    }
}
