package gov.va.nvap.web.user;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author Zack Peterson
 */
public class User implements Serializable {
    
    private static final long serialVersionUID = 1L;

    private String name;
    private List<String> roles;
    private ArrayList<UserPreference> userPreferences = new ArrayList<UserPreference>();

    public User() {}

    public User(String name) {
        this.name = name;
    }

    public User(String name, List<String> roles) {
        this.name = name;
        this.roles = roles;
    }

    public User(List<String> roles) {
        this.roles = roles;
    }

    public boolean hasName() {
        return !(name == null || name.isEmpty() || name.equals("null"));
    }

    public boolean hasRole(String roleName) {
        return roles != null && roles.contains(roleName);
    }

    public void addRole(String roleName) {
        if (roles == null) roles = new ArrayList<>();

        roles.add(roleName);
    }

    public boolean hasRoles() {
        return !(roles == null || roles.isEmpty());
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public List<String> getRoles() {
        return roles;
    }

    public void setRoles(List<String> roles) {
        this.roles = roles;
    }

    public List<UserPreference> getUserPreferences() {
        return userPreferences;
    }

    public void setUserPreferences(ArrayList<UserPreference> userPreferences) {
        this.userPreferences = userPreferences;
    }
    
    public void addUserPreference(String key, String value) {
        //i'm a smart method. i check to see if a user already has the preference passed in and update it otherwise i add it.
        if(this.hasUserPreference(key)) {
            for(UserPreference up : getUserPreferences()) {
                if(key.equals(up.getKey())) {
                    up.setValue(value);
                    break;
                }
            }
        } else {
            UserPreference newUP = new UserPreference(key, value);
            getUserPreferences().add(newUP);
        }
    }
    
    public void removeUserPreference(String key) {
        for(UserPreference up : getUserPreferences()) {
            if(key.equals(up.getKey())) {
                getUserPreferences().remove(up);
                break;
            }
        }
    }
    
    public boolean hasUserPreference(String key) {
        boolean hasPref = false;
        
        for(UserPreference up : getUserPreferences()) {
            if(key.equals(up.getKey())) {
                hasPref = true;
                break;
            }
        }
            
        return hasPref;
    }
    
    public String getUserPreference(String key) {
        String userPref = "";
        
        for(UserPreference up : getUserPreferences()) {
            if(key.equals(up.getKey())) {
                userPref = up.getValue();
                break;
            }
        }
        
        return userPref;
    }
}
