package gov.va.nvap.web.user.document;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

/**
 *
 * @author Johann P. Sonnenberg
 * 
 * This bean represents a scheduled export created (and managed) by a user of VAP
 */

@Entity
@Table(name = "USER_DOCUMENT_SCHEDULE")
public class UserDocumentSchedule implements Serializable {
    private static final long serialVersionUID = 1L;
    
    @Id
    @GeneratedValue(strategy = GenerationType.AUTO)
    @Basic(optional = false)
    @Column(name = "SCHEDULE_ID")
    private Long scheduleId;
    
    @Basic(optional = false)
    @Column(name = "USER_ID")
    private String userId;
    
    @Basic(optional = false)
    @Column(name = "CREATION_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date creationDate;
    
    @Basic(optional = false)
    @Column(name = "LAST_UPDATED_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date lastUpdatedDate;
    
    @Basic(optional = false)
    @Column(name = "REPORT_TYPE")
    private String reportType;
    
    @Basic(optional = false)
    @Column(name = "USER_DEFINED_REPORT_NAME")
    private String userDefinedReportName;
    
    @Basic(optional = false)
    @Column(name = "FORMAT")
    private String format;
    
    @Basic(optional = false)
    @Column(name = "PERIOD_OF_REPORTING")
    private String periodOfReporting;
    
    @Lob
    @Column(name = "REPORT_PARAMETERS")
    private String reportParameters;
    
    @Lob
    @Column(name = "REPORT_VIEW_PARAMETERS")
    private String reportViewParameters;
            
    @Basic(optional = false)
    @Column(name = "FREQUENCY")
    private String frequency;
    
    @Basic(optional = true)
    @Column(name = "WEEKLY_RECURRENCE_WEEK")
    private Long weeklyRecurrenceWeek;
    
    @Basic(optional = true)
    @Column(name = "WEEKLY_RECURRENCE_DAY_OF_WEEK")
    private Long weeklyRecurrenceDayOfWeek;
    
    @Basic(optional = true)
    @Column(name = "MONTHLY_RECURRENCE_MONTH")
    private Long monthlyRecurrenceMonth;
    
    @Basic(optional = false)
    @Column(name = "MONTHLY_RECURRENCE_DAY")
    private String monthlyRecurrenceDay;
    
    @Basic(optional = false)
    @Column(name = "START_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date startDate;
    
    @Basic(optional = false)
    @Column(name = "END_DATE")
    @Temporal(TemporalType.TIMESTAMP)
    private Date endDate;

    @Column(insertable = false, updatable = false, name = "FREQUENCY_TEXT")
    //@Column(name = "FREQUENCY_TEXT")
    private String frequencyText;
    
    public UserDocumentSchedule() {
    }

    public UserDocumentSchedule(Long scheduleId) {
        this.scheduleId = scheduleId;
    }
    
    //NOTE: all constructors except for the "everything" constructor, do NOT include the reportParameters and reportViewParameters. this is by design as creating thsoe properties takes more care/time. 
    
    //default constructor (gets the bean everything but the reportParameters and what the daily/weekly/monthly recurrence settings are)
    public UserDocumentSchedule(String userId, Date creationDate, Date lastUpdatedDate, String reportType, String userDefinedReportName, String format, String periodOfReporting, String frequency, Date startDate, Date endDate) {
        this.userId = userId;
        this.creationDate = creationDate;
        this.lastUpdatedDate = lastUpdatedDate;
        this.reportType = reportType;
        this.userDefinedReportName = userDefinedReportName;
        this.format = format;
        this.periodOfReporting = periodOfReporting;
        this.frequency = frequency;
        this.startDate = startDate;
        this.endDate = endDate;
    }

    //daily recurrence constructor
    public UserDocumentSchedule(String userId, Date creationDate, Date lastUpdatedDate, String reportType, String userDefinedReportName, String format, String periodOfReporting, String frequency, Date startDate, Date endDate, String frequencyText) {
        this.userId = userId;
        this.creationDate = creationDate;
        this.lastUpdatedDate = lastUpdatedDate;
        this.reportType = reportType;
        this.userDefinedReportName = userDefinedReportName;
        this.format = format;
        this.periodOfReporting = periodOfReporting;
        this.frequency = frequency;
        this.startDate = startDate;
        this.endDate = endDate;
        this.frequencyText = frequencyText;
    }
    
    //weekly recurrence constructor
    public UserDocumentSchedule(String userId, Date creationDate, Date lastUpdatedDate, String reportType, String userDefinedReportName, String format, String periodOfReporting, String frequency, Long weeklyRecurrenceWeek, Long weeklyRecurrenceDayOfWeek, Date startDate, Date endDate, String frequencyText) {
        this.userId = userId;
        this.creationDate = creationDate;
        this.lastUpdatedDate = lastUpdatedDate;
        this.reportType = reportType;
        this.userDefinedReportName = userDefinedReportName;
        this.format = format;
        this.periodOfReporting = periodOfReporting;
        this.frequency = frequency;
        this.weeklyRecurrenceWeek = weeklyRecurrenceWeek;
        this.weeklyRecurrenceDayOfWeek = weeklyRecurrenceDayOfWeek;
        this.startDate = startDate;
        this.endDate = endDate;
        this.frequencyText = frequencyText;
    }
    
    //monthly recurrence constructor
    public UserDocumentSchedule(String userId, Date creationDate, Date lastUpdatedDate, String reportType, String userDefinedReportName, String format, String periodOfReporting, String frequency, Date startDate, Date endDate, Long monthlyRecurrenceMonth, String monthlyRecurrenceDay, String frequencyText) {
        this.userId = userId;
        this.creationDate = creationDate;
        this.lastUpdatedDate = lastUpdatedDate;
        this.reportType = reportType;
        this.userDefinedReportName = userDefinedReportName;
        this.format = format;
        this.periodOfReporting = periodOfReporting;
        this.frequency = frequency;
        this.startDate = startDate;
        this.endDate = endDate;
        this.monthlyRecurrenceMonth = monthlyRecurrenceMonth;
        this.monthlyRecurrenceDay = monthlyRecurrenceDay;
        this.frequencyText = frequencyText;
    }
    
    //the "everything" constructor
    public UserDocumentSchedule(Long scheduleId, String userId, Date creationDate, Date lastUpdatedDate, String reportType, String userDefinedReportName, String format, String periodOfReporting, String reportParameters, String reportViewParameters, String frequency, Long weeklyRecurrenceWeek, Long weeklyRecurrenceDayOfWeek, Long monthlyRecurrenceMonth, String monthlyRecurrenceDay, Date startDate, Date endDate, String frequencyText) {
        this.scheduleId = scheduleId;
        this.userId = userId;
        this.creationDate = creationDate;
        this.lastUpdatedDate = lastUpdatedDate;
        this.reportType = reportType;
        this.userDefinedReportName = userDefinedReportName;
        this.format = format;
        this.periodOfReporting = periodOfReporting;
        this.reportParameters = reportParameters;
        this.reportViewParameters = reportViewParameters;
        this.frequency = frequency;
        this.weeklyRecurrenceWeek = weeklyRecurrenceWeek;
        this.weeklyRecurrenceDayOfWeek = weeklyRecurrenceDayOfWeek;
        this.monthlyRecurrenceMonth = monthlyRecurrenceMonth;
        this.monthlyRecurrenceDay = monthlyRecurrenceDay;
        this.startDate = startDate;
        this.endDate = endDate;
        this.frequencyText = frequencyText;
    }

    public Long getScheduleId() {
        return scheduleId;
    }

    public void setScheduleId(Long scheduleId) {
        this.scheduleId = scheduleId;
    }

    public String getUserId() {
        return userId;
    }

    public void setUserId(String userId) {
        this.userId = userId;
    }

    public Date getCreationDate() {
        return creationDate;
    }

    public void setCreationDate(Date creationDate) {
        this.creationDate = creationDate;
    }

    public Date getLastUpdatedDate() {
        return lastUpdatedDate;
    }

    public void setLastUpdatedDate(Date lastUpdatedDate) {
        this.lastUpdatedDate = lastUpdatedDate;
    }

    public String getReportType() {
        return reportType;
    }

    public void setReportType(String reportType) {
        this.reportType = reportType;
    }

    public String getUserDefinedReportName() {
        return userDefinedReportName;
    }

    public void setUserDefinedReportName(String userDefinedReportName) {
        this.userDefinedReportName = userDefinedReportName;
    }

    public String getFormat() {
        return format;
    }

    public void setFormat(String format) {
        this.format = format;
    }

    public String getPeriodOfReporting() {
        return periodOfReporting;
    }

    public void setPeriodOfReporting(String periodOfReporting) {
        this.periodOfReporting = periodOfReporting;
    }
    
    public String getReportParameters() {
        return reportParameters;
    }

    public void setReportParameters(String reportParameters) {
        this.reportParameters = reportParameters;
    }

    public String getReportViewParameters() {
        return reportViewParameters;
    }

    public void setReportViewParameters(String reportViewParameters) {
        this.reportViewParameters = reportViewParameters;
    }

    public String getFrequency() {
        return frequency;
    }

    public void setFrequency(String frequency) {
        this.frequency = frequency;
    }

    public Long getWeeklyRecurrenceWeek() {
        return weeklyRecurrenceWeek;
    }

    public void setWeeklyRecurrenceWeek(Long weeklyRecurrenceWeek) {
        this.weeklyRecurrenceWeek = weeklyRecurrenceWeek;
    }

    public Long getWeeklyRecurrenceDayOfWeek() {
        return weeklyRecurrenceDayOfWeek;
    }

    public void setWeeklyRecurrenceDayOfWeek(Long weeklyRecurrenceDayOfWeek) {
        this.weeklyRecurrenceDayOfWeek = weeklyRecurrenceDayOfWeek;
    }
    
    public Long getMonthlyRecurrenceMonth() {
        return monthlyRecurrenceMonth;
    }

    public void setMonthlyRecurrenceMonth(Long monthlyRecurrenceMonth) {
        this.monthlyRecurrenceMonth = monthlyRecurrenceMonth;
    }

    public String getMonthlyRecurrenceDay() {
        return monthlyRecurrenceDay;
    }

    public void setMonthlyRecurrenceDay(String monthlyRecurrenceDay) {
        this.monthlyRecurrenceDay = monthlyRecurrenceDay;
    }

    public Date getStartDate() {
        return startDate;
    }

    public void setStartDate(Date startDate) {
        this.startDate = startDate;
    }

    public Date getEndDate() {
        return endDate;
    }

    public void setEndDate(Date endDate) {
        this.endDate = endDate;
    }

    public String getFrequencyText() {
        return frequencyText;
    }

    public void setFrequencyText(String frequencyText) {
        this.frequencyText = frequencyText;
    }
    
    //helper methods
    
    public Object getReportParameterValue(String key) throws JSONException {
        //gets the value of a key/value pair out of the reportParameters. returns an empty string if the key is not found.
        try {
            JSONObject json = new JSONObject(this.getReportParameters());
        
            return json.get(key);
        } catch( JSONException e ) {
            return "";
        }
    }
    
    public List getReportParameterArrayValue(String key) throws JSONException {
        //gets the value (in a List of objects) out of the key/value pair of reportParameters. returns an empty List if key is not found.
        try {
            JSONObject json = new JSONObject(this.getReportParameters());
            JSONArray arr = json.getJSONArray(key);
            
            List<Object> returnList = new ArrayList<Object>();
            for(int i=0; i < arr.length(); i++) {
                returnList.add(arr.get(i));
            }
            
            return returnList;
        } catch( JSONException e ) {
            return new ArrayList<Object>();
        }
    }
}