<%-- 
    Document   : default view for the manage scheduled exports. initial exports are those the user has created.
    Created on : Feb 19, 2018, 11:07:29 AM
    Author     : Johann P. Sonnenberg
--%>

<%@taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c" %>
<%@taglib tagdir="/WEB-INF/tags/formatting" prefix="f" %>
<%@taglib uri="http://java.sun.com/jsp/jstl/functions" prefix="fn" %>

<f:mainTemplate headerText="Advanced Scheduling">
    <script language="JavaScript">
        $(document).ready(function(){
            VAP.common.setProcessingForDataTable('reportTable', 'divReportPanel');

            VAP.common.reportTable = $('#reportTable').DataTable( {
                destroy: true,
                searching: false,
                processing: true,
                serverSide: true,
                order: [[4, 'desc']],
                ajax: "ManageScheduledExports.do_sec?getUserExports=true",
                columnDefs: [
                    {orderable: false, targets: [0, 7]},
                    {className: "vertical-align-center", targets: 0}
                ],
                lengthMenu: [ 10, 25, 50, 100, 250, 500 ],
                pageLength: 25
            });
            
            VAP.common.reportTable.on('draw', function () {
                VAP.common.addAriaToDataTable('reportTable', VAP.common.reportTable);
            });
            
            //handler for onchange of the checkboxes 
            $(document).on('change', '.userScheduledExportActionCheckbox', function(){
                //alert that a checkbox was checked or not checked
                var status = $(this).is(":checked") ? true : false;
                var alertMessage = $(this).attr('aria-label');
                
                if(status) {
                    VAP.common.fireAriaAlert(alertMessage + ' checked.');
                } else {
                    VAP.common.fireAriaAlert(alertMessage + ' not checked.');
                }
                
                if($('input[name=userScheduledExportAction]:checked').length > 0) {
                    $('#removeButton').removeClass('disabled');
                    $('#removeButton').attr('aria-label', 'Remove, Active');
                    $('#removeButton').attr('title', 'Remove, Active');
                } else {
                    $('#removeButton').addClass('disabled');
                    $('#removeButton').attr('aria-label', 'Remove, Inactive');
                    $('#removeButton').attr('title', 'Remove, Inactive');
                }
            });
            
            //handler for the select all
            $("#master").change(function(){
                var status = $(this).is(":checked") ? true : false;
                $(".userScheduledExportActionCheckbox").prop("checked",status);
                
                if(status) {
                    VAP.common.fireAriaAlert('Select All checkbox checked.');
                } else {
                    VAP.common.fireAriaAlert('Select All checkbox not checked.');
                }
                
                if($('input[name=userScheduledExportAction]:checked').length > 0) {
                    $('#removeButton').removeClass('disabled');
                    $('#removeButton').attr('aria-label', 'Remove, Active');
                    $('#removeButton').attr('title', 'Remove, Active');
                } else {
                    $('#removeButton').addClass('disabled');
                    $('#removeButton').attr('aria-label', 'Remove, Inactive');
                    $('#removeButton').attr('title', 'Remove, Inactive');
                }
            });
            
            
            
            $('#refreshButton').on('click', function() {
                $('#reportTable .dataTables_empty').html(''); //blank out the empty table message if there is one
                $('#reportTable').DataTable().ajax.reload();
                
                //disable the remove button again
                $('#removeButton').addClass('disabled');
                $('#removeButton').attr('aria-label', 'Remove, Inactive');
                $('#removeButton').attr('title', 'Remove, Inactive');
            });
            
            $('#removeButton').on('click', function(){
                if(!$(this).hasClass('disabled')) {
                   //pop the confirm removal modal since the button is NOT disabled
                   $('#confirmRemoveUserScheduledExportModal').modal('show');
                }
            });
            
            $('#confirmRemoveUserScheduledExportModal').on('show.bs.modal', function (event) {
                var removalText = 'this (1) task';
                var howManyExports = $('input[name=userScheduledExportAction]:checked').length;
                
                if(howManyExports > 1) {
                    removalText = 'these (' + howManyExports + ') tasks';
                }
                
                $('#removalText').html(removalText);
            });
            
            //handle the delete
            $('#btnConfirmRemoveUserScheduledExport').on('click', function(){
                //loop over and create a list of exports to delete
                var removalIds = $('input[name=userScheduledExportAction]:checked').map(function() { 
                    return $(this).attr('id'); 
                }).get().join(',');

                //if a message is still showing, hide it
                $('#userScheduledExportMessage').hide();
                //set it's class back to success
                $('#userScheduledExportMessage').removeClass('alert-danger');
                $('#userScheduledExportMessage').addClass('alert-success');
                
                //make the ajax call to delete them
                $.ajax({
                    url: "ManageScheduledExports.do_sec?removeUserScheduledExportAttempt=true&ids=" + removalIds, 
                    headers: { 'skipLoadingAlert' : 'true' },
                    success: function (result) {
                        var info = $.parseJSON(result);

                        //refresh the table
                        $('#reportTable .dataTables_empty').html(''); //blank out the empty table message if there is one
                        $('#reportTable').DataTable().ajax.reload();

                        //pop the success message
                        $('#userScheduledExportMessageBody').html('Successfully removed ' + $('input[name=userScheduledExportAction]:checked').length + ' task(s).');
                        $('#userScheduledExportMessage').show().delay(5000).slideUp();
                        
                        //disable the remove button again
                        $('#removeButton').addClass('disabled');
                        $('#removeButton').attr('aria-label', 'Remove, Inactive');
                        $('#removeButton').attr('title', 'Remove, Inactive');
                    }, 
                    error: function(result) {
                       $('#userScheduledExportMessage').addClass('alert-danger');
                       $('#scheduledExportErrorMessageBody').html('There was an error removing the selected export(s). Please try again later.');
                       $('#scheduledExportErrorMessage').show(); 
                    }
                }); 
            });
            
            <c:if test="${not empty advancedSchedulingMessage}">
                $('#userScheduledExportMessageBody').html('<c:out value="${advancedSchedulingMessage}" />');
                $('#userScheduledExportMessage').show().delay(5000).slideUp();
            </c:if>
        });
    </script>
    <div class="row">
        <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
            <div class="alert alert-info">
                The Advanced Scheduling functionality allows users to set scheduled exports to run in advance by selecting the report, frequency, format, and end dates for the task recurrence. 
                This page displays a list of all the existing tasks scheduled to run and is unique by user. To delete a recurring task, select the "Remove" button. To create a new advanced 
                scheduled task, navigate to the desired report under the "Detailed Reports" main menu. The exported files can be retrieved through the "Scheduled Exports" page.
            </div>
            <div id="userScheduledExportMessage" class="alert alert-success pad-bottom-10 pad-top-10 pad-left-10" role="alert" style="display: none;">
                <button type="button" class="close" data-dismiss="alert" aria-label="Close"><span aria-hidden="true">&times;</span></button>
                <span id="userScheduledExportMessageBody"></span>
            </div>
        </div>
    </div>
    <div><div aria-live="assertive" role="alert" id="reportTable_info"></div></div>
    <div id="divReportContent" class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
        <div class="panel panel-default" id="divReportPanel">
            <div class="panel-body">
                <div class="pull-right">
                    <button class="btn btn-danger disabled" id="removeButton" title="Remove, Inactive" aria-label="Remove, Inactive">Remove <span class="fa fa-trash" aria-hidden="true"></span></button>
                    <button class="btn" id="refreshButton" title="Refresh">Refresh <span class="fa fa-refresh" aria-hidden="true"></span></button>
                </div>
                <table id="reportTable" class="display table table-striped" width="100%" cellspacing="0" role="presentation">
                    <thead>
                        <tr role="row">
                            <th role="columnheader" style="width: 80px;" class="js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingSelectAllColumn']}"/>"><span id="selectAllHeader" aria-live="assertive" aria-atomic="true" data-simpletooltip-target="tableheader"><input type="checkbox" class="vertical-align-bottom" name="master" id="master" aria-label="Select All Rows" /> <span aria-label="Select All Rows " id="selectAllSpan">Select All</span></span></th>
                            <th role="columnheader" class="text-center js-simple-tooltip" style="min-width: 220px;" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingNameColumn']}"/>"><span data-simpletooltip-target="tableheader">Task Name</span></th>
                            <th role="columnheader" class="text-center js-simple-tooltip" style="min-width: 220px;" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingTypeColumn']}"/>"><span data-simpletooltip-target="tableheader">Report Type</span></th>
                            <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingFormatColumn']}"/>"><span data-simpletooltip-target="tableheader">Format</span></th>
                            <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingDateCreatedColumn']}"/>"><span data-simpletooltip-target="tableheader">Date Created (CT)</span></th>
                            <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingStartDateColumn']}"/>"><span data-simpletooltip-target="tableheader">Start Date</span></th>
                            <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary-left" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingEndDateColumn']}"/>"><span data-simpletooltip-target="tableheader">End Date</span></th>
                            <th role="columnheader" class="text-center js-simple-tooltip" data-simpletooltip-prefix-class="secondary-left" data-simpletooltip-text="<c:out value="${descriptors['advancedSchedulingFrequencyColumn']}"/>"><span data-simpletooltip-target="tableheader">Frequency</span></th>
                        </tr>
                    </thead>
                </table>
            </div>
        </div>
    </div>
    
    <div class="modal fade" id="confirmRemoveUserScheduledExportModal" role="dialog" data-backdrop="static" aria-labelledby="confirmRemoveUserScheduledExportModalLabel">
        <div class="vertical-alignment-helper">
            <div class="modal-dialog vertical-align-center" role="document">
                <div class="modal-content" tabindex="-1">
                    <div class="modal-header">
                        <h4 class="modal-title" id="confirmRemoveUserScheduledExportModalLabel">Confirm Remove Scheduled Task(s)</h4>
                    </div>
                    <div class="modal-body" aria-live="assertive" role="alert">
                        Are you sure you want to permanently remove <span id="removalText"></span>? This cannot be undone. 
                    </div>
                    <div class="modal-footer">
                        <button type="button" class="btn btn-default" data-dismiss="modal">Cancel</button>
                        <button type="button" id="btnConfirmRemoveUserScheduledExport" class="btn btn-primary" data-dismiss="modal">Confirm</button>
                    </div>
                </div>
            </div>
        </div>
    </div>
</f:mainTemplate>