<%@taglib uri="http://java.sun.com/jsp/jstl/core" prefix="c"%>
<%@taglib uri="http://java.sun.com/jsp/jstl/functions" prefix="fn"%>
<%@taglib uri="http://java.sun.com/jsp/jstl/fmt" prefix="fmt"%>
<%@taglib tagdir="/WEB-INF/tags/cpp" prefix="cpp"%>
<%@taglib tagdir="/WEB-INF/tags/date" prefix="date"%>
<%@taglib tagdir="/WEB-INF/tags/formatting" prefix="f" %>
<c:if test="${param.consentTypeShort eq 'wft'}">
    <%-- dealing with Withhold for Treatment Revocation (opt-out) --%>
    <c:set var="revokeNoun" value="Rescind of Opt-Out"/>
    <c:set var="authFormName" value="TBD_OPTOUT"/>
    <c:set var="consentType" value="Withhold for Treatment Revocation"/>
</c:if>

<c:if test="${param.consentTypeShort eq 'nwhin7332'}">
    <c:set var="revokeNoun" value="Revocation"/>
    <c:set var="consentType" value="NwHIN Revocation"/>
    <c:choose>
        <c:when test="${enableShareNormalWithoutConsentModel}">
            <%-- new model where only 38 USC &#xa7; 7332 requires opt-in --%>
            <c:set var="authFormName" value="TBD_7332"/>
        </c:when>
        <c:otherwise>
            <%-- old require opt-in model --%>
            <c:set var="authFormName" value="10-0484"/>
        </c:otherwise>
    </c:choose>
</c:if>

<script>
    $(document).ready(function () {        
        $('#signatureDate').datepicker({
            theme: 'vap',
            inputFormat: 'MM/dd/yyyy',
            outputFormat: 'MM/dd/yyyy',
            max: '<c:out value="${today}" />',
            gainFocusOnConstruction: false
        });

        $('#deceasedDate').datepicker({
            theme: 'vap',
            inputFormat: 'MM/dd/yyyy',
            outputFormat: 'MM/dd/yyyy',
            max: '<c:out value="${today}" />',
            gainFocusOnConstruction: false
        });
        
        //call the calendar validation helper tag to remove the error classes for the date inputs and surrounding divs when tabbing/clicking off of the fields
        <date:calendarValidationHelper elementsToRemoveErrorClassesOn="revokeDateAndUploadValidationFormGroup,signatureDate,revokeDateAndUploadValidationPatientDeceasedFormGroup,deceasedDate" />
        
        //because this page is generated dynamically, we need to set up the validator via JavaScript
        $('#frmRevokeNwHIN').validator({
            custom: {
                isValidDateMMDDYYYY: function($el) { 
                    var today = moment();
                    var dateToValidate = moment($el.val(), 'MM/DD/YYYY');
                    
                    //return true to denote an error
                    return !moment($el.val(), 'MM/DD/YYYY', true).isValid() || dateToValidate.isAfter(today);
                }
            }
        });
    });
</script>
<div class="row">
    <div class="col-xs-12 col-sm-12 col-md-12 col-lg-12">
        <div class="alert alert-info">
            <c:if test="${param.consentTypeShort eq 'wft'}">
                <%-- dealing with Withhold for Treatment Revocation (opt-out) --%>
                <p>
                    The Veteran has currently requested to opt-out (withhold the release of protected health information)
                    from non-VA health care provider organizations participating in the <c:out value="${organizationName}" />
                    and partnering with VA for treatment purposes. Rescinding this request to opt-out
                    will result in the VA immediately sharing Veteran electronic health information, unless
                    that information contains 38 USC &#xa7; 7332 Protected conditions and the
                    Veteran has not specifically authorized sharing such protected data through the <c:out value="${organizationName}" />.
                </p>
                <p>
                    Rescind opt-out (rescind withholding the release of protected health information)
                    from <c:out value="${organizationName}"/> for the purposes of treatment.
                </p>
            </c:if>

            <c:if test="${param.consentTypeShort eq 'nwhin7332'}">
                <c:choose>
                    <c:when test="${enableShareNormalWithoutConsentModel}">
                        <%-- new model where only 38 USC &#xa7; 7332 requires opt-in --%>
                        <p>
                            The Veteran's current Authorization permits VA to share his/her
                            38 USC &#xa7; 7332 Protected electronic health information with non-VA health care provider
                            organizations participating in the <c:out value="${organizationName}" /> and partnering with VA
                            for treatment purposes. Revocation of this authorization will result
                            in the VA no longer having permission to share 38 USC &#xa7; 7332 Protected electronic health
                            information in this way. If the Veteran revokes this Authorization,
                            VA will stop sharing his/her 38 USC &#xa7; 7332 Protected electronic health information with private
                            health care provider through <c:out value="${organizationName}" />.
                        </p>
                    </c:when>
                    <c:otherwise>
                        <%-- old require opt-in model --%>
                        <p>
                            The Veteran's current Authorization permits VA to share his/her
                            electronic health information with non-VA health care provider
                            organizations participating in the <c:out value="${organizationName}" /> and partnering with VA
                            for treatment purposes. Revocation of this authorization will result
                            in the VA no longer having permission to share electronic health
                            information in this way. If the Veteran revokes this Authorization,
                            VA will stop sharing his/her electronic health information with private
                            health care provider through <c:out value="${organizationName}" />.
                        </p>
                    </c:otherwise>
                </c:choose>
                <p>
                    To revoke this authorization, check the box and enter the Patient Signature Date and click <strong>Revoke</strong>.
                </p>
                <p>
                    To submit a new authorization, select New Authorization as the 'Inactivation Reason' 
                    and enter the Patient Signature Date for the new authorization form received.
                </p>
            </c:if>
            <p>
                Note: VA patient information is always shared with the DoD,
                a patient's Authorization/Revocation status does not affect interactions with
                the Department of Defense. 
            </p>
        </div>
    </div>
</div>
<div id="revokeNwHINErrorMessage" class="alert alert-danger" style="display: none;" role="alert"></div>
<div>
    <form name="frmRevokeNwHIN" id="frmRevokeNwHIN" action="PatientDetails.do_sec" method="POST" class="form-horizontal" data-toggle="validator" role="form">
        <input type="hidden" name="icn" value="<c:out value="${patientDemographics.icn}" />" /> 
        <input type="hidden" name="requestId" value="<c:out value="${requestId}" />" /> 
        <input type="hidden" name="patientId" value="<c:out value="${consentDirective.patientIen}" />" /> 
        <c:if test="${param.consentTypeShort eq 'wft'}">
            <input type="hidden" name="consentType" value="Withhold for Treatment Revocation" /> 
        </c:if>
        <c:if test="${param.consentTypeShort eq 'nwhin7332'}">
            <input type="hidden" name="consentType" value="NwHIN Revocation" /> 
        </c:if>
        <input type="hidden" name="purposeOfUse" value="TREATMENT" />
        <input type="hidden" name="expirationYears" value="10" />
        <input type="hidden" name="optOut" value="true" /><%-- this is the method in the servlet this form goes to, you know, just in case it's ambiguous... --%>
        <input type="hidden" name="signatureDateHolder" id="signatureDateHolder" value="" />
        <input type="hidden" name="deceasedDateHolder" id="deceasedDateHolder" value="" />

        <div class="form-group">
            <label for="authenticatingFacility" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label"><f:requiredField altText="Authenticating Facility is a required field." labelText="Authenticating Facility" /></label> 
            <div class="col-xs-9 col-sm-9 col-md-9 col-lg-9">
                <select id="authenticatingFacility" name="userFacility" class="form-control" required>
                    <c:forEach var="f" items="${allowedFacilities}">
                        <option value="<c:out value="${f.facilityStation}" />"
                            <c:choose>
                                <c:when test="${f.facilityStation eq defaultUserFacility.facilityStation}">
                                    selected
                                </c:when>
                                <c:when test="${cookie.facility.value == f.facilityStation}">
                                    selected
                                </c:when>
                            </c:choose>>
                            <c:choose>
                                <c:when test="${f.facilityName != ''}">
                                    <c:out value="${f.facilityName}" />
                                </c:when>
                                <c:otherwise>
                                    <c:out value="${fn:split(f.facilityDns, '.')[0]}" />
                                </c:otherwise>
                            </c:choose>
                            (<c:out value="${f.facilityStation}" />)
                        </option>
                    </c:forEach>
                </select> 
            </div>
        </div>
        <div id="revokeForm">
            <div class="form-group">
                <label for="reason" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label"><f:requiredField altText="Inactivation Reason is a required field." labelText="Inactivation Reason" /></label> 
                <div class="col-xs-4 col-sm-4 col-md-4 col-lg-4">
                    <select id="reason" name="reason" onchange="onOptoutReasonChange(this);" onkeyup="onOptoutReasonChange(this);" class="form-control" required>
                        <option title="Select a Opt out reason from the dropdown list" value="">--Select a reason--</option>
                        <c:if test="${!empty optOutReasons}">
                            <c:forEach var="r" items="${optOutReasons}" varStatus="i">
                                <c:choose>
                                    <c:when test="${r eq 'New Authorization'}">
                                        <c:if test="${(isNewAuthorizationAllowed eq true) and (param.consentTypeShort eq 'nwhin7332')}">
                                            <%-- Withhold for Treatment doesn't expire, so no need to renew. --%>
                                            <option value="<c:out value="${r}" />"><c:out value="${r}" /></option>
                                        </c:if>
                                    </c:when>
                                    <c:when test="${(r eq 'Authorization Expired') and (param.consentTypeShort eq 'wft')}">
                                        <%-- Withhold for Treatment doesn't expire, so don't output. --%>
                                    </c:when>
                                    <c:otherwise>
                                        <option value="<c:out value="${r}" />"><c:out value="${r}" /></option>
                                    </c:otherwise>
                                </c:choose>
                            </c:forEach>
                        </c:if>
                    </select>
                </div>
            </div>
            <fieldset>
                <legend class="vap-input-group-legend"></legend>
                <div id="revokeValidationCheckBox" style="display: none;" class="col-xs-offset-3 col-sm-offset-3 col-md-offset-3 col-lg-offset-3 pad-bottom-10">
                    <div class="checkbox form-group margin-left-10">
                        <label>
                            <input id="revokeValidationCheckBoxInput"  type="checkbox" title="select Form Validation checkbox if ${revokeNoun} Form ${authFormName} was received and validated. required field" name="revokeValidationCheckBoxInput" value="${revokeNoun} Form Received and Validated">
                            <f:requiredField altText="${revokeNoun} Form Received and Validated is a required field." labelText="Form Validation for ${revokeNoun} Form ${authFormName} Received and Validated" />
                        </label>
                    </div>
                </div>
                <div id="newValidationCheckBox" style="display: none;" class="col-xs-offset-3 col-sm-offset-3 col-md-offset-3 col-lg-offset-3 pad-bottom-10">
                    <div class="checkbox form-group margin-left-10">
                        <label>
                            <input id="newValidationCheckBoxInput" type="checkbox" title="select Form Validation checkbox if New Authorization Form 10-0485 was received and validated. required field" name="newValidationCheckBoxInput" value="New Authorization Form Received and Validated">
                            New Authorization Form 10-0485 Received and Validated
                        </label>
                    </div>
                </div>
            </fieldset>
            <div id="revokeDateAndUploadValidation" style="display: none;">
                <div class="form-group" id="revokeDateAndUploadValidationFormGroup">
                    <label for="signatureDate" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label"><f:requiredField altText="Patient Signature Date is a required field." labelText="Patient Signature Date" /></label> 
                    <div class="col-xs-4 col-sm-4 col-md-4 col-lg-4">
                        <input type="text" name="signatureDate" id="signatureDate" class="form-control vap-datepicker-input" data-isValidDateMMDDYYYY="isValidDateMMDDYYYY" />
                    </div>
                </div>
            </div>
            <div id="revokeDateAndUploadValidationPatientDeceased" style="display: none;">
                <div class="form-group" id="revokeDateAndUploadValidationPatientDeceasedFormGroup">
                    <label for="deceasedDate" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label"><f:requiredField altText="Patient Deceased Date is a required field." labelText="Patient Deceased Date" /></label> 
                    <div class="col-xs-4 col-sm-4 col-md-4 col-lg-4">
                        <input type="text" name="deceasedDate" id="deceasedDate" class="form-control vap-datepicker-input" data-isValidDateMMDDYYYY="isValidDateMMDDYYYY" />
                    </div>
                </div>
            </div>
            <div class="form-group">
                <label for="comments" class="col-xs-3 col-sm-3 col-md-3 col-lg-3 control-label">Comments</label>
                <div class="col-xs-9 col-sm-9 col-md-9 col-lg-9">
                    <textarea id="comments" name="comments" class="form-control" rows="5"></textarea>
                </div>
            </div>
        </div>
    </form>
</div>