#!/bin/sh

echo "Installing Desktop"
yum -y groupinstall "Desktop" "Desktop Platform" "X Window System" "Fonts"
yum -y install xorg-x11-apps xterm zip unzip vim bc net-tools

echo "Adding oracle user."
groupadd dba
useradd oracle -g dba
mkdir -p /u01/app
chown -R oracle:dba /u01/app

echo "Adding vagrant to dba group."
usermod -a -G dba vagrant

echo "Downloading installers..."
if [ ! -d /vap/.vagrant/Software/vap ]; then
  mkdir -p /vap/.vagrant/Software/vap
  curl https://downloads.vetsez.net/Downloads/jdk-8u121-linux-x64.tar.gz -o /vap/.vagrant/Software/vap/jdk-8u121-linux-x64.tar.gz
  curl https://downloads.vetsez.net/Downloads/fmw_12.2.1.2.0_wls_Disk1_1of1.zip -o /vap/.vagrant/Software/vap/fmw_12.2.1.2.0_wls_Disk1_1of1.zip
  curl https://downloads.vetsez.net/Downloads/oracle-xe-11.2.0-1.0.x86_64.rpm.zip -o /vap/.vagrant/Software/vap/oracle-xe-11.2.0-1.0.x86_64.rpm.zip
fi

echo "Installing JDK."
runuser -l oracle << STOP
mkdir -p /u01/app/oracle/java
tar -C /u01/app/oracle/java -xzf /vap/.vagrant/Software/vap/jdk-8u121-linux-x64.tar.gz
pushd /u01/app/oracle/java
ln -s jdk1.8.0_121 latest
popd
STOP

echo "Instaling weblogic"
runuser -l oracle << STOP
echo "  Creating oraInst.loc."
mkdir -p /u01/app/oracle/oraInventory
cat << STOP2 >/u01/app/oracle/oraInventory/oraInst.loc
inventory_loc=/u01/app/oracle/oraInventory
inst_group=dba
STOP2
echo "  Running WebLogic Installer."
export JAVA_HOME=/u01/app/oracle/java/latest
pushd /tmp
unzip /vap/.vagrant/Software/vap/fmw_12.2.1.2.0_wls_Disk1_1of1.zip
/u01/app/oracle/java/latest/bin/java -Djava.security.egd=file:/dev/./urandom -jar fmw_12.2.1.2.0_wls.jar -silent -responseFile /vap/vagrant-scripts/wls.rsp -invPtrLoc /u01/app/oracle/oraInventory/oraInst.loc
popd
echo "  Configure WebLogic to use /dev/urandom."
cat << STOP2 >>/u01/app/oracle/weblogic/oracle_common/common/bin/commBaseEnv.sh
# use /dev/urandom to generate random numbers to speed up weblogic.
# change made here so that everything we use in WL installation
# can benefit.
export JAVA_OPTIONS="\\\$JAVA_OPTIONS -Djava.security.egd=file:/dev/./urandom"
export CONFIG_JVM_ARGS="\\\$CONFIG_JVM_ARGS -Djava.security.egd=file:/dev/./urandom"
STOP2
STOP

echo "Installing Oracle XE"
# trick XE installer into thinking we have 2GB of swap space.
mv /usr/bin/free /usr/bin/free.bak
printf "#!/bin/sh\necho Swap - - 2048" >/usr/bin/free
chmod +x /usr/bin/free
# end trick
unzip /vap/.vagrant/Software/vap/oracle-xe-11.2.0-1.0.x86_64.rpm.zip -d /tmp/XE
pushd /tmp/XE/Disk1
rpm -ivh oracle-xe-11.2.0-1.0.x86_64.rpm >/tmp/XE/install.log
ORA_INSTALL_STATUS=$?
/etc/init.d/oracle-xe configure responseFile=/vap/vagrant-scripts/xe.rsp >>/tmp/XE/install.log
popd
if [ $ORA_INSTALL_STATUS -ne 0 ]; 
then
    echo -e "\nOracle RDBMS 11g was not installed successfully. There are some problems installing it.\nLook into the log for more details, /tmp/XE/install.log. \nExiting...."
    exit $ORA_INSTALL_STATUS
else 
    echo -e "Oracle RDBMS 11g was installed successfully. ORA_INSTALL_STATUS=$ORA_INSTALL_STATUS. \n"
fi
# undo trick
rm -f /usr/bin/free
mv /usr/bin/free.bak /usr/bin/free
# end undo trick

echo "Add Java, Oracle and WebLogic to paths."
tee -a ~vagrant/.bash_profile ~oracle/.bash_profile << STOP
export JAVA_HOME=/u01/app/oracle/java/latest
export WL_HOME=/u01/app/oracle/weblogic
export PATH=\$WL_HOME/oracle_common/common/bin:\$JAVA_HOME/bin:\$PATH
. /u01/app/oracle/product/11.2.0/xe/bin/oracle_env.sh
STOP
tee -a ~vagrant/.bashrc ~oracle/.bashrc << STOP
alias debugwl='cd /u01/app/domains/vap_devl && ./startDomainDebug.sh && tail -f servers/vap_devl_admin/logs/vap_devl.out'
alias debugwlm='cd /u01/app/domains/vap_devl && ./startDomainDebugWithMocks.sh && tail -f servers/vap_devl_admin/logs/vap_devl.out'
alias stopwl='cd /u01/app/domains/vap_devl && bin/stopWebLogic.sh'
alias logwl='cd /u01/app/domains/vap_devl && tail -f servers/vap_devl_admin/logs/vap_devl.out'
STOP

echo "Give everyone in dba group write privs on /u01/app."
chmod -R g+w /u01/app

echo -e "\nSet up the VAP schema.  Look in /tmp/BuildDB.log for log output."
runuser -l oracle << STOP
pushd /vap/DBScripts
. ./BuildDB.sh &>/tmp/BuildDB.log
popd
STOP

echo "Set up vap_devl weblogic domain."
runuser -l vagrant << STOP
wlst.sh /vap/vagrant-scripts/vap_devl.py
cp /vap/vagrant-scripts/weblogic/* /u01/app/domains/vap_devl
STOP
